/*
  Copyright (c) 2000,2001 Al Sutton (al@alsutton.com)
  All rights reserved.
  Redistribution and use in source and binary forms, with or without modification, are permitted
  provided that the following conditions are met:

  1. Redistributions of source code must retain the above copyright notice, this list of conditions
  and the following disclaimer.

  2. Redistributions in binary form must reproduce the above copyright notice, this list of
  conditions and the following disclaimer in the documentation and/or other materials provided with
  the distribution.

  Neither the name of AlSutton.com nor the names of its contributors may be used to endorse or promote
  products derived from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
  OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package com.alsutton.filesync;

import java.io.*;
import java.util.*;

/**
 * Class holding the information about a directory.
 */

public class DirectoryInformation
  extends Synchronizable
{
  /**
   * Default constructor. Protected for internal use only.
   */

  protected DirectoryInformation()
  {
  }

  /**
   * Constructor. Stores the directory name relative to the base.
   *
   * @param _filename The relative filename.
   */

  public DirectoryInformation( String base, File theFile )
  {
    super( base, theFile );
  }

  /**
   * Constructor, Reconstructs the data from strea,.
   *
   * @param stream The stream holding the data.
   */

  public void read( InputStream stream )
    throws IOException
  {
    int stringLength = 0;

    int nextByte;
    for( int i = 0 ; i < 2 ; i++ )
    {
      stringLength <<= 8;

      nextByte = stream.read();
      if( nextByte == -1 )
        throw new IOException( "End of stream encountered before data finished" );

      stringLength += ((int)nextByte & 0xff );
    }

    byte[] stringBytes = new byte[ stringLength ];
    int counter = 0;
    while( counter < stringLength )
    {
      counter += stream.read(stringBytes, counter, stringLength-counter);
    }
    filename = new String( stringBytes );
  }

  /**
   * Method to write this data to a stream.
   *
   * @param stream The stream to write it to.
   */

  public void write( OutputStream stream )
    throws IOException
  {
    byte[] data = new byte[2];

    int tempData = filename.length();
    for( int i = 1 ; i >= 0 ; i-- )
    {
      data[i] = (byte)(tempData & 0xff);
      tempData >>= 8;
    }
    stream.write(data);

    byte[] stringData = filename.getBytes();
    stream.write(stringData);
  }

  /**
   * Method to compare this object to an object in the given base
   * directory.
   *
   * @param newBase The new base to check the file against.
   * @return true if the file is the same, false otherwise.
   */

  public boolean checkAgainstBase( String newBase )
  {
    String fullFilename = newBase + File.separatorChar + filename;
    File testFile = new File( fullFilename );

    if( testFile.exists() == false )
      return false;

    if( testFile.isDirectory() == false )
      return false;

    return true;
  }

  /**
   * Method to get the file from a specified base and send it over a stream.
   *
   * @param base The base for the file to be retrieved.
   * @param stream The stream over which it should be sent.
   */

  public void sendData( String base, OutputStream stream )
    throws IOException
  {
    // Nothing to do because directories store no data in themselves.
  }

  /**
   * Method to receive and create the file in a new base area.
   *
   * @param base The new base area to receive it into.
   * @param stream The stream over which to accept the file.
   */

  public void receiveData( String base, InputStream stream )
    throws IOException
  {
    String fullDirectoryname = base + File.separatorChar + filename;
    File directory = new File( fullDirectoryname );
    directory.delete();
    directory.mkdir();
  }
}